import org.apache.commons.math3.ode.*;

/** Wrapper used to pass a <i>system of first order ordinary differential equations</i> (odes) to <a target="_blank" href="http://commons.apache.org/math">Commons Math</a> algorithms.
  * Computes the derivatives {@code dy}<sub>i</sub>{@code /dt} defined as {@code String[] odes}.<br/>
  * <br/>
  * See <a target="_top" href="../0_CommonsMath_README.html#examples">CommonsMath_ODE_Lotka_Volterra~Test.jxn</a>
  * for an example.  
  */
public class JxnODE implements FirstOrderDifferentialEquations
{
   KmgFormelInterpreter fi;
   static final boolean skipInputLogEnabled = true;
   
   String itsInit = "";
   String [] itsOdes;

   /** Constructs a {@link org.apache.commons.math3.ode.FirstOrderDifferentialEquations}
     * instance to be used by <a target="_blank" href="http://commons.apache.org/math">Commons Math</a> algorithms. <br/>
     * Example:
     * <pre>
     *    ode = @JxODE( $this, { "$y[0]*( a - b $y[1] );", "$y[1]*( d $y[0] - c );" } )
     * </pre>
     * @param fi instance of {@code KmgFormelInterpreter} on which the definitions in {@code odes[...]} are executed
     * @param odes array of <i>statement_sequence</i>s (one or more statements separated by '{@code ;}').
     * The definitions in {@code odes[...]} must use {@code double[] $y} and {@code $t} to calculate the derivatives {@code dy/dt}. 
     * If the definition in {@code odes[i]} ends with '{@code ;}', log output of intermediate steps is suppressed for the definition.
     */
   public JxnODE( KmgFormelInterpreter fi, String [] odes )
   {
      this.fi = fi;
      itsOdes = odes;
   }
   
   /** See {@link #JxnODE}.
     * @param init <i>statement_sequence</i> (one or more statements separated by '{@code ;}'). {@code init} is evaluated before evaluating {@code odes} and
     * can use {@code double[] $y} and {@code $t}. If {@code init} ends with '{@code ;}', log output is suppressed.
     * @param parameters the values can be used in the {@code init} and {@code odes} definitions via the array variable {@code $p[...]}
     */
   public JxnODE( KmgFormelInterpreter fi, String init, String [] odes, double [] parameters )
   {
      this.fi = fi;
      itsInit = init;
      itsOdes = odes;
      
      fi.put( "$p", parameters );
   }
   
// --- interface FirstOrderDifferentialEquations ---

   /** Returns the number of states. */
   public int getDimension()
   {
      return itsOdes.length;
   }

   /** Computes derivatives {@code ydot} from {@code y} and {@code t} (internally called by Commons Math algorithms). */
   public void computeDerivatives( double t, double [] y, double [] ydot )
   {
   // store values
      fi.put( "$t", new JxnPrimitiveWrapper(t) );
      fi.put( "$y", y );
      if( itsInit.length() > 0 ) fi.execute( itsInit, skipInputLogEnabled );
      
   // calculate derivatives
      for( int i = 0; i < itsOdes.length; i++ )
      {
         ydot[i] = KmgFormelInterpreter.doubleValue( fi.execute( itsOdes[i], skipInputLogEnabled ) );
      }    
   }
   
   public String toString()
   {
      return itsOdes.length + " ODEs";
   }
}
